<?php

namespace FiloBlu\Flow\Model\From\Entity;

use Exception;
use FiloBlu\Flow\Helper\LoggerProvider;
use Magento\Catalog\Model\ProductFactory;
use Magento\Eav\Model\Config;
use Magento\Eav\Model\ConfigFactory;
use Magento\Eav\Model\Entity\Attribute\Source\BooleanFactory;
use Magento\Eav\Model\Entity\AttributeFactory;
use Magento\Eav\Model\Entity\TypeFactory;
use Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\CollectionFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Registry;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Model\Customer as CustomerModel;
use Magento\Customer\Api\Data\CustomerInterfaceFactory AS CustomerFactory;
use Magento\Customer\Api\Data\CustomerInterface;
use FiloBlu\Flow\Helper\Data as HelperData;

/**
 * Class Customer
 * @package FiloBlu\Flow\Model\From\Entity
 */
class Customer extends AbstractEntity
{

    const EAV_CODE_PASSWORD_HASH = 'password_hash';
    const EAV_CODE_PASSWORD = 'password';
    /**
     * @var ConfigFactory
     */
    protected $configFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $_customerRepositoryInterface;
    /**
     * @var CustomerFactory
     */
    protected $customerFactory;
    /**
     * @var bool
     */
    public $isCreate = false;

    protected $helperData;

    /**
     * Customer constructor.
     * @param Registry $registry
     * @param ProductFactory $productFactory
     * @param \FiloBlu\Flow\Helper\Product $helperProduct
     * @param \FiloBlu\Flow\Helper\Attribute $helperAttribute
     * @param LoggerProvider $loggerProvider
     * @param AttributeFactory $attributeFactory
     * @param CollectionFactory $attributeOptionCollectionFactory
     * @param ObjectManagerInterface $objectManager
     * @param ConfigFactory $configFactory
     * @param TypeFactory $entityTypeFactory
     * @param BooleanFactory $booleanFactory
     * @param CustomerRepositoryInterface $customerRepositoryInterface
     * @param CustomerFactory $customerFactory
     * @param HelperData $helperData
     * @param array $data
     */
    public function __construct(
        Registry $registry,
        ProductFactory $productFactory,
        \FiloBlu\Flow\Helper\Product $helperProduct,
        \FiloBlu\Flow\Helper\Attribute $helperAttribute,
        LoggerProvider $loggerProvider,
        AttributeFactory $attributeFactory,
        CollectionFactory $attributeOptionCollectionFactory,
        ObjectManagerInterface $objectManager,
        ConfigFactory $configFactory,
        TypeFactory $entityTypeFactory,
        BooleanFactory $booleanFactory,
        CustomerRepositoryInterface $customerRepositoryInterface,
        CustomerFactory $customerFactory,
        HelperData $helperData,
        array $data = []
    ) {
        parent::__construct(
            $registry,
            $productFactory,
            $helperProduct,
            $helperAttribute,
            $loggerProvider,
            $attributeFactory,
            $attributeOptionCollectionFactory,
            $objectManager,
            $entityTypeFactory,
            $booleanFactory,
            $data
        );
        $this->configFactory = $configFactory;
        $this->_customerRepositoryInterface = $customerRepositoryInterface;
        $this->customerFactory = $customerFactory;
        $this->helperData = $helperData;
    }

    /**
     * @return bool|mixed|null
     * @throws Exception
     */
    public function getObjectId()
    {
        return $this->_objectEntityId;
    }

    protected function getObject(){
        $email = $this->entityIdentifier;
        $store_id = $this->storeId;
        $store = $this->helperData->getStoreFromId($store_id);
        try {
            $customer = $this->_customerRepositoryInterface->get($email,$store->getWebsiteId());
        } catch (\Exception $e){
            $customer = $this->customerFactory->create();
            $this->isCreate = true;
        }
        $this->_objectEntityId = $customer->getId();
        return $customer;
    }

    protected function setObjectAttributeValue()
    {
        $attribute_code = $this->getAttribute()->getAttributeCode();
        $attributeValue = $this->getAttributeValue();
        $object = $this->getObject();
        if (in_array($attribute_code,array(self::EAV_CODE_PASSWORD,self::EAV_CODE_PASSWORD_HASH)) && !$this->isCreate()){
            // Do not change password on update customer
            return $object;
        }

        $object->setData($attribute_code,$attributeValue);
        $object->setCustomAttribute($attribute_code,$attributeValue);
        $this->setMandatoryData($object);

        try {
            $saved_customer = $this->_customerRepositoryInterface->save($object);
        } catch (\Exception $e){

        }

        return $object;
    }

    protected function setMandatoryData($object){
        $firstname = $object->getFirstname();
        $lastname = $object->getLastname();
        $websiteId = $object->getWebsiteId();
        if (!$firstname){
            $object->setFirstname('null');
        }
        if (!$lastname){
            $object->setLastname('null');
        }
        if ($this->storeId && !$object->getStoreId()){
            $object->setStoreId($this->storeId);
        }
        if (!$object->getEmail()){
            $object->setEmail($this->entityIdentifier);
        }
        return $object;
    }

    public function isCreate(){
        return $this->isCreate;
    }

}
